@@ -9,6 +9,10 @@
  * Part of HTML-XML-utils, see:
  * http://www.w3.org/Tools/HTML-XML-utils/
  *
+ * TO DO: Should it be an option whether URL references of the form
+ * "", "#foo" and "?bar" are replaced by "oldurl", "oldurl#foo" and
+ * "oldurl?bar"? (See adjust_url().)
+ *
  * Created: 5 Dec 2008
  * Author: Bert Bos <bert@w3.org>
  *
@@ -83,9 +87,12 @@ static string path_from_url_to_url(const conststring a, const conststring b)
 
 
 /* adjust_url -- return a new URL relative to newurl instead of oldurl */
-static string adjust_url(const conststring url)
+static conststring adjust_url(const conststring url)
 {
-  return URL_s_absolutize(newbase, url);
+  if (!url || !url[0] || url[0] == '#' || url[0] == '?')
+    return url;			/* Don't replace references to self */
+  else
+    return URL_s_absolutize(newbase, url);
 }
 
 
@@ -162,7 +169,7 @@ void handle_pi(void *clientdata, string pi_text)
 /* handle_starttag -- called after a start tag is parsed */
 void handle_starttag(void *clientdata, string name, pairlist attribs)
 {
-  string v;
+  conststring v;
   pairlist p;
 
   fprintf(out, "<%s", name);
@@ -185,7 +192,7 @@ void handle_starttag(void *clientdata, string name, pairlist attribs)
 /* handle_emptytag -- called after an empty tag is parsed */
 void handle_emptytag(void *clientdata, string name, pairlist attribs)
 {
-  string v;
+  conststring v;
   pairlist p;
 
   fprintf(out, "<%s", name);
