@@ -2,7 +2,7 @@
  * incl - expand included files
  *
  * Searches for <!--include "file"--> and expands the referenced file
- * in place. File may be a URL. Works recursively. Other acepted
+ * in place. File may be a URL. Works recursively. Other accepted
  * syntaxes:
  *
  * <!--include "file"-->
@@ -19,7 +19,7 @@
  *
  * Author: Bert Bos
  * Created: 2 Dec 1998
- * Version: $Id: hxincl.c,v 1.3 2008/11/28 22:45:56 bbos Exp $
+ * Version: $Id: hxincl.c,v 1.5 2009/03/25 18:46:04 bbos Exp $
  *
  **/
 #include <config.h>
@@ -43,6 +43,7 @@
 #include <assert.h>
 #include "export.h"
 #include "types.e"
+#include "errexit.e"
 #include "html.e"
 #include "scan.e"
 #include "dict.e"
@@ -66,6 +67,8 @@ static Boolean final = False;	/* Final means to remove the comments */
 static Boolean has_error = False;
 static string base = NULL;
 static stack skipping = NULL;
+static Dictionary substitutions = NULL;
+
 
 
 /* push -- push a skipping state on the stack */
@@ -106,6 +109,20 @@ static Key word_to_key(const string s, int len)
   return KNone;
 }
 
+/* add_substitution -- add a file name substitution to the dictionary */
+static void add_substitution(const string assignment)
+{
+  string s;
+
+  /* To do: handle file names containing '='. Add escapes? */
+  if (!substitutions) substitutions = dict_create(10);
+  if (!substitutions) errexit("Out of memory.\n");
+  if (!(s = strchr(assignment, '='))) errexit("No '=' found in option -s\n");
+  *s = '\0';
+  if (!dict_add(substitutions, assignment, s + 1)) errexit("Out of memory?\n");
+  *s = '=';
+}
+
 /* --------------- implements interface api.h -------------------------- */
 
 /* handle_error -- called when a parse error occurred */
@@ -136,7 +153,7 @@ void handle_comment(void *clientdata, string commenttext)
   /* A push() occurs at <!--begin-include...--> and at include_file() */
   /* A pop() occurs at <!--end-include...--> and at ENDINCL */
   int i, j;
-  string s, url;
+  conststring s, url;
   FILE *f;
   Key key;
 
@@ -180,6 +197,9 @@ void handle_comment(void *clientdata, string commenttext)
       url = newnstring(commenttext + i, j);
     }
 
+    /* If we have a substitution for it, use that instead */
+    if ((s = dict_find(substitutions, url))) url = s;
+
     /* Get the file and recursively parse it */
     s = URL_s_absolutize(base, url);
     if (!(f = fopenurl(s, "r")))
@@ -279,14 +299,14 @@ void handle_endincl(void *clientdata)
 /* usage -- print usage message and exit */
 static void usage(string prog)
 {
-  fprintf(stderr, "Version %s\nUsage: %s [-x] [-b base] [file-or-url]\n",
+  fprintf(stderr, "Version %s\nUsage: %s [-x] [-b base] [-s name=subst ...] [file-or-url]\n",
 	  VERSION, prog);
   exit(2);
 }
 
 int main(int argc, char *argv[])
 {
-  int i;
+  int c;
 
   /* Bind the parser callback routines to our handlers */
   set_error_handler(handle_error);
@@ -302,27 +322,26 @@ int main(int argc, char *argv[])
   set_endincl_handler(handle_endincl);
 
   /* Parse command line arguments */
-  for (i = 1; i < argc && argv[i][0] == '-' && !eq(argv[i], "--"); i++) {
-    switch (argv[i][1]) {
-      case 'x': do_xml = True; break;
-      case 'b': base = argv[++i]; break;
-      case 'f': final = True; break;
-      default: usage(argv[0]);
+  while ((c = getopt(argc, argv, ":xb:fs:")) != -1)
+    switch (c) {
+    case 'x': do_xml = True; break;
+    case 'b': base = optarg; break;
+    case 'f': final = True; break;
+    case 's': add_substitution(optarg); break;
+    default: usage(argv[0]);
     }
-  }
-  if (i < argc && eq(argv[i], "--")) i++;
 
-  if (i == argc) {
+  if (optind == argc) {
     yyin = stdin;
     base = ".";
-  } else if (i == argc - 1) {
-    yyin = fopenurl(argv[i], "r");
-    if (!base) base = argv[i];
+  } else if (optind == argc - 1) {
+    yyin = fopenurl(argv[optind], "r");
+    if (!base) base = argv[optind];
   } else {
     usage(argv[0]);
   }
 
-  if (yyin == NULL) {perror(argv[i]); exit(1);}
+  if (yyin == NULL) {perror(argv[optind]); exit(1);}
 
   if (yyparse() != 0) exit(3);
 
