@@ -37,7 +37,7 @@
  *
  * Author: Bert Bos <bert@w3.org>
  * Created: 11 Apr 2000
- * Version: $Id: hxindex.c,v 1.5 2009/01/19 19:26:43 bbos Exp $
+ * Version: $Id: hxindex.c,v 1.6 2009/03/25 18:46:19 bbos Exp $
  *
  **/
 #include <config.h>
@@ -361,7 +361,7 @@ static void mkindex(Indexterm terms)
   string p, h;
 
   printf("<ul class=\"indexlist\">");
-  globalprevious = "";
+  globalprevious = "zzzzzzzzzzzzzzzzzz"; /* An unlikely alphanumeric string */
   twalk(terms, write_index_item);
 
   /* Close all open lists */
