@@ -6,7 +6,7 @@
  *
  * Created 9 May 1998
  * Bert Bos <bert@w3.org>
- * $Id: hxnormalize.c,v 1.2 2009/01/19 19:23:04 bbos Exp $
+ * $Id: hxnormalize.c,v 1.4 2009/03/08 15:24:00 bbos Exp $
  */
 #include <config.h>
 #include <stdio.h>
@@ -143,8 +143,7 @@ static Boolean needs_quotes(const string s)
   assert(s);
   if (!s[0]) return True;			/* Empty string */
   for (i = 0; s[i]; i++)
-    if (!isalnum(s[i]) && (s[i] != '-') && (s[i] != '_')
-	&& (s[i] != '.') && (s[i] != ':')) return True;
+    if (!isalnum(s[i]) && (s[i] != '-') && (s[i] != '.')) return True;
   return False;
 }
 
