@@ -65,15 +65,16 @@ static int http_proxy_init = 0, ftp_proxy_init = 0;
 
 /* Forward declaration */
 FILE *fopenurl2(const conststring path, const conststring mode,
-		const Dictionary headers, Dictionary response);
+		const Dictionary headers, Dictionary response,
+		int maxredirs);
 
 
 /* open_http2 -- open resource via HTTP; return file pointer or NULL */
 static FILE *open_http2(const URL server, const conststring url,
-			Dictionary headers, Dictionary response)
+			Dictionary headers, Dictionary response, int maxredirs)
 {
   Boolean delete_response = !response, delete_headers = !headers;
-  int fd, n, redircount;
+  int fd, n;
   conststring h, v;
   char buf[BUFLEN];
   string s;
@@ -81,6 +82,11 @@ static FILE *open_http2(const URL server, const conststring url,
   conststring machine = server->machine;
   conststring port = server->port ? server->port : "80";
 
+  if (maxredirs < 1) {
+    errno = ELOOP;
+    return NULL;
+  }
+
   /* Connect */
   fd = connectTCP(machine, port);
   if (fd == -1) return NULL;
@@ -126,23 +132,13 @@ static FILE *open_http2(const URL server, const conststring url,
 	     hasprefix(buf+9, "303") || hasprefix(buf+9, "307")) {
     fclose(f);
     if (!(v = dict_find(response, "location"))) {
+      errno = 121;		/* EREMOTEIO */
       f = NULL;			/* Redirect without a location!? */
     } else {
       s = newstring(v);		/* Because we'll delete the response dict. */
-      if (!headers) headers = dict_create(2);
-      if ((h = dict_find(headers, " redirects")) &&
-	  (redircount = atoi(h)) >= MAXREDIRECTS) {
-	f = NULL;		/* Too many redirects */
-      } else {
-	(void) sprintf(buf, "%d", h ? ++redircount : 1);
-	if (!dict_add(headers, " redirects", buf)) {
-	  f = NULL;		/* Out of memory */
-	} else {
-	  dict_destroy_all(response);
-	  f = fopenurl2(s, "r", headers, response); /* Redirect */
-	  dispose(s);
-	}
-      }
+      dict_destroy_all(response);
+      f = fopenurl2(s, "r", headers, response, maxredirs--); /* Redirect */
+      dispose(s);
     }
   } else if (!hasprefix(buf + 9, "200")) {
     if (hasprefix(buf + 9, "404")) errno = EFAULT;
@@ -160,7 +156,8 @@ static FILE *open_http2(const URL server, const conststring url,
 
 
 /* open_http -- open resource via HTTP; return file pointer or NULL */
-static FILE *open_http(const URL url, Dictionary headers, Dictionary response)
+static FILE *open_http(const URL url, Dictionary headers, Dictionary response,
+		       int maxredirs)
 {
   string proxy;
 
@@ -177,12 +174,14 @@ static FILE *open_http(const URL url, Dictionary headers, Dictionary response)
     http_proxy_init = 1;
   }
 
-  return open_http2(http_proxy?http_proxy:url, url->full, headers, response);
+  return open_http2(http_proxy ? http_proxy : url, url->full, headers,
+		    response, maxredirs);
 }
 
 
 /* open_ftp -- open resource via FTP; return file pointer or NULL */
-static FILE *open_ftp(const URL url, Dictionary headers, Dictionary response)
+static FILE *open_ftp(const URL url, Dictionary headers, Dictionary response,
+		      int maxredirs)
 {
   string proxy;
 
@@ -190,7 +189,8 @@ static FILE *open_ftp(const URL url, Dictionary headers, Dictionary response)
     if ((proxy = getenv("ftp_proxy"))) ftp_proxy = URL_new(proxy);
     ftp_proxy_init = 1;
   }
-  if (ftp_proxy) return open_http2(ftp_proxy, url->full, headers, response);
+  if (ftp_proxy)
+    return open_http2(ftp_proxy, url->full, headers, response, maxredirs);
 
   /* Can only work via proxy for now... */
   errno = ENOSYS;
@@ -200,7 +200,7 @@ static FILE *open_ftp(const URL url, Dictionary headers, Dictionary response)
 
 /* open_file -- open resource as local file or FTP; return file ptr or NULL */
 static FILE *open_file(const URL url, const conststring mode,
-		       Dictionary headers, Dictionary response)
+		       Dictionary headers, Dictionary response, int maxredirs)
 {
   FILE *f = NULL;
 
@@ -209,7 +209,7 @@ static FILE *open_file(const URL url, const conststring mode,
   }
   if (! f) {
     if (! eq(mode, "r")) errno = EACCES;	/* Not yet supported */
-    else f = open_ftp(url, headers, response);
+    else f = open_ftp(url, headers, response, maxredirs);
   }
   return f;
 }
@@ -217,7 +217,8 @@ static FILE *open_file(const URL url, const conststring mode,
 
 /* fopenurl2 -- like fopenurl, but sends and returns HTTP headers */
 EXPORT FILE *fopenurl2(const conststring path, const conststring mode,
-		       const Dictionary headers, Dictionary response)
+		       const Dictionary headers, Dictionary response,
+		       int maxredirs)
 {
   URL url;
   FILE *f = NULL;
@@ -229,12 +230,12 @@ EXPORT FILE *fopenurl2(const conststring path, const conststring mode,
     f = fopen(path, mode);			/* Assume it's a local file */
   } else if (eq(url->proto, "http")) {
     if (! eq(mode, "r")) errno = ENOSYS;	/* Not yet supported */
-    else f = open_http(url, headers, response);
+    else f = open_http(url, headers, response, maxredirs);
   } else if (eq(url->proto, "ftp")) {
     if (! eq(mode, "r")) errno = ENOSYS;	/* Not yet supported */
-    else f = open_ftp(url, headers, response);
+    else f = open_ftp(url, headers, response, maxredirs);
   } else if (eq(url->proto, "file")) {
-    f = open_file(url, mode, headers, response);
+    f = open_file(url, mode, headers, response, maxredirs);
   } else {
     errno = EACCES;				/* Unimplemented protocol */
   }
@@ -244,7 +245,7 @@ EXPORT FILE *fopenurl2(const conststring path, const conststring mode,
 
 
 /* fopenurl -- like fopen, but takes a URL; HTTP headers are parsed */
-EXPORT FILE *fopenurl(const string path, const string mode)
+EXPORT FILE *fopenurl(const conststring path, const conststring mode)
 {
-  return fopenurl2(path, mode, NULL, NULL);
+  return fopenurl2(path, mode, NULL, NULL, MAXREDIRECTS);
 }
